<?php

declare(strict_types=1);

namespace app\common\model;

use think\exception\HttpException;

use app\common\model\Base;
use think\facade\Config;

class Article extends Base
{
    protected $schema = [
        "id"          => "int",
        "cid"         => "int",
        "title"       => "varchar",
        "writer"      => "varchar",
        "source"      => "varchar",
        "titlepic"    => "varchar",
        "keywords"    => "varchar",
        "summary"     => "varchar",
        "content"     => "varchar",
        "discussed"   => "int",
        "status"      => "int",
        "top"         => "int",
        "sort"       => "int",
        "hits"        => "int",
        "likes"       => "int",
        "content_type"=> "int",
        "userid"      => "int",
        "username"    => "varchar",
        "create_time" => "int",
        "update_time" => "int"
    ];

    protected $disuse = ['top','discussed'];

    public function category()
    {
        return $this->belongsTo('Category', 'cid')->bind(['category_name' => 'name', 'category_url' => 'url', 'route' => 'route']);
    }

    public static function queryPage($params)
    {
        $where = [];

        if (isset($params['status'])) {
            $where[] = ['status', '=', (int) $params['status']];
        }

        if (!empty($params['cid'])) {
            $where[] = ['cid', '=', (int) $params['cid']];
        }

        if (!empty($params['key'])) {
            $where[] = ['title|keywords', 'LIKE', '%' . (string)$params['key'] . '%'];
        }

        if (!empty($params['yearMonth'])) {
            if (!preg_match("/^([0-9]{4})\/([0-9]{1,2})$/", $params['yearMonth']) && !preg_match("/^([0-9]{4})-([0-9]{1,2})$/", $params['yearMonth'])) {
                throw new HttpException(404, '日期格式不正确');
            }
            $separator = strrpos('/', $params['yearMonth']) ? '/' : '-';
            
            $year = $month = '';
            
            if (strnatcasecmp(PHP_VERSION, '7.0.0') >= 0) {
                list($year, $month) = explode($separator, $params['yearMonth']);
            } else {
                list($month, $year) = explode($separator, $params['yearMonth']);
            }
            
            if ((int) $month < 1 || (int) $month > 12) {
                throw new HttpException(404, '日期格式不正确');
            }
            
            $days = month_frist_and_last_day($year, $month);
            
            // $where['create_time'] = ['between', [$days['firstday'], $days['lastday']]];
            $where[] = ['create_time','between', [$days['firstday'], $days['lastday']]];
        }

        $limit = empty($params['limit']) ? Config::get('app.page_size', 20) : (int)$params['limit'];
        
        $order = ['id desc'];
        if (!empty($params['order'])) {
            if (is_array($params['order'])) {
                $order = array_unique(array_merge($params['order']));
            } else {
                array_push($order, $params['order']);
                $order = array_unique($order);
            }
        }

        $list = self::where($where)
            ->field('id,cid,title,titlepic,username,summary,content_type,hits,sort,status,create_time')
            ->with(['category'])
            ->order($order)->paginate($limit, false, ['query' => $params]);

        return $list;
    }

    public static function getListByCid($cid = 0, $limit = 10, $order = [])
    {
        $where = [];

        if ($cid != 0) {
            if ($cid < 0) {
                $where[] = ['cid', '<>', abs($cid)];
            } else {
                $where[] = ['cid', '=', $cid];
            }
        }

        $order = !empty($order) ? array_merge($order, ['id' => 'desc']) : ['id' => 'desc'];

        return self::where($where)
            ->field('id,cid,title,titlepic,username,summary,hits,sort,status,create_time')
            ->order($order)->limit($limit)->select();
    }

    public static function getTop($limit)
    {
        return self::with('category')->field('id,cid,title,titlepic,summary,username,hits,sort,status,create_time')
            ->order('sort desc')->limit($limit)->select();
    }

    public static function getOne($id)
    {
        if (!$id) {
            throw new HttpException(404, '页面不存在');
        }

        return self::with('category')->find($id);
    }

    public static function getNextPrev($id, $cid = null)
    {
        if ($cid) {
            $id_list = self::where(['status' => 1, 'cid' => $cid])
                ->order(['sort' => 'desc', 'id' => 'desc'])
                ->column('id');
        } else {
            $id_list = self::where(['status' => 1])->column('id');
        }

        if (count($id_list)==1) {
            $data_p = null;
            $data_N = null;
        } else {
            $key = array_search($id, $id_list);
            if ($key == 0) {
                $data_p = null;
                $N = $id_list[1];
                $data_N = self::field('id,title,create_time')->find($N);
            } elseif ($key == count($id_list) - 1) {
                $P = $id_list[count($id_list) - 2];
                $data_p = self::field('id,title,create_time')->find($P);
                $data_N = null;
            } else {
                $P = $id_list[$key - 1];
                $data_p =  self::field('id,title,create_time')->find($P);
                $N = $id_list[$key + 1];
                $data_N =  self::field('id,title,create_time')->find($N);
            }
        }

        return ['prev' => $data_p, 'next' => $data_N];
    }

    public static function createTimeArchive()
    {
        $create_times = self::order('create_time desc')->column('create_time');
        $timeList = [];
        foreach ($create_times as $value) {
            $yearAndMonth = date("Y-m", $value);
            $timeList[] = $yearAndMonth;
        }
        $timeList = array_unique($timeList);

        return $timeList;
    }
}
